﻿using Borepin.Model;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Xamarin.Essentials;

namespace Borepin.Service.ServerInstances
{
    /// <summary>
    /// Save connected BFFHInstances in Xamarin.Essentials.Preferences
    /// </summary>
    public class BFFHInstanceService : IBFFHInstanceService
    {
        private List<BFFHInstance> _BFFHInstance_List;

        public Task<List<BFFHInstance>> GetBFFHInstances()
        {
            List<BFFHInstance> bffhInstances = JsonConvert.DeserializeObject<List<BFFHInstance>>(Preferences.Get("bffhinstances", "[]"));

            _BFFHInstance_List = bffhInstances;

            return Task.FromResult(bffhInstances);
        }

        public Task<bool> AddBFFHInstance(BFFHInstance instance)
        {
            if(_BFFHInstance_List == null)
            {
                GetBFFHInstances();
            }

            if(_BFFHInstance_List.Find(x => x.Address == instance.Address) != null)
            {
                return Task.FromResult(true);
            }

            _BFFHInstance_List.Add(instance);

            SaveList();

            return Task.FromResult(true);
        }
        
        public Task<bool> UpdateBFFHInstance(BFFHInstance instance)
        {
            if (_BFFHInstance_List == null)
            {
                GetBFFHInstances();
            }

            if (_BFFHInstance_List.Find(x => x.Address == instance.Address) == null)
            {
                throw new ArgumentException("BFFH Instance does not exist");
            }

            foreach (BFFHInstance bffh in _BFFHInstance_List.Where(x => x.Address == instance.Address))
            {
                bffh.Name = instance.Name;
                bffh.Description = instance.Description;
            }

            SaveList();

            return Task.FromResult(true);
        }

        private Task<bool> SaveList()
        {
            Preferences.Set("bffhinstances", JsonConvert.SerializeObject(_BFFHInstance_List));

            return Task.FromResult(true);
        }

        public Task<bool> RemoveBFFHInstance(BFFHInstance instance)
        {
            if (_BFFHInstance_List == null)
            {
                GetBFFHInstances();
            }

            _BFFHInstance_List.RemoveAll(x => x.Address == instance.Address);

            SaveList();

            return Task.FromResult(true);
        }

        public Task<bool> RemoveAllBFFHInstances()
        {
            Preferences.Remove("bffhinstances");
            _BFFHInstance_List.Clear();

            return Task.FromResult(true);
        }
    }
}
